using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazorise;
using Blazorise.DataGrid;
using Microsoft.AspNetCore.Components;
using Microsoft.JSInterop;
using UserService.Infrastructure;
using UserService.Infrastructure.DataModels;

namespace UserService.Pages
{
    public abstract class MembersBase<TMember> : ComponentBase where TMember : Member
    {
        [Inject] protected IJSRuntime JsRuntime { get; set; } = null!;

        protected IReadOnlyList<OrganizationUnit>? OrganizationUnits { get; set; }

        protected IReadOnlyList<TMember>? Members { get; set; }

        protected string? CustomFilterValue { get; set; }

        protected abstract Task RowInsertingCallback(CancellableRowChange<TMember, Dictionary<string, object>> arg);

        protected abstract Task RowInsertedCallback(SavedRowItem<TMember, Dictionary<string, object>> arg);
        protected abstract Task RowDeletingCallback(CancellableRowChange<TMember> arg);
        protected abstract Task RowDeletedCallback(TMember user);

        protected abstract Task RowUpdatingCallback(CancellableRowChange<TMember, Dictionary<string, object>> arg);
        protected abstract bool OnCustomFilter(TMember model);

        protected void ValidateCommonName(ValidatorEventArgs e)
        {
            if (e == null) throw new ArgumentNullException(nameof(e));
            var commonName = e.Value?.ToString();
            var validationResult = Validators.ValidateCommonName(commonName, Members ?? Enumerable.Empty<Member>());
            e.Status = validationResult == false ? ValidationStatus.Error : ValidationStatus.Success;
        }
    }
}